import numpy as np

class Circle:
    '''A class for circles
      Constructor is initialized with two tuples, one for the center of the circle
      and the other for a point on the circle.
      
      Methods include radius, area, and circum.  None of these methods accept any arguments.
      
      The user is not required to pre-compute the radius of the circle.  Exception testing is 
      done in area and circum to check for a circle radius.  If it doesn't exist, a radius is 
      computed.
    '''
    
    def __init__(self, center, point):
        self.xc = center[0]
        self.yc = center[1]
        self.x = point[0]
        self.y = point[1]
    
    def radius(self):
        x = self.x - self.xc
        y = self.y - self.yc
        self.R = np.sqrt(x * x + y * y)
    
    def area(self):
        try:
            self.A = np.pi * self.R* self.R
        except AttributeError:
            # If radius hasn't been defined yet
            x = self.x - self.xc
            y = self.y - self.yc
            r = np.sqrt(x * x + y * y)
            self.R = r
            self.A = np.pi * r * r
    
    def circum(self):
        try:
            self.C =  2.0 * np.pi * self.R
        except AttributeError:
            # If radius hasn't been defined yet
            x = self.x - self.xc
            y = self.y - self.yc
            r = np.sqrt(x * x + y * y)
            self.R = r
            self.C = 2.0 * np.pi * r


if __name__ == "__main__":
    centered_at = (1.0, 1.0) # Circle center
    through_point = (2.0, 4.0) # A point on the circle
    
    C1 = Circle(centered_at, through_point)
    
    C1.radius()
    C1.area()
    C1.circum()
    
    print("This circle has radius {0:17.16f}.".format(C1.R))
    print("This circle has area {0:17.16f}.".format(C1.A))
    print("This circle has circumference {0:17.16f}.".format(C1.C))
